#include "ecdsweda.h"
#include <QDebug>

const QString libraryName("ecdSweda");


struct stAddress
{
    DWORD Ip;   //!< Um inteiro (32 bits) sem sinal (DWORD) que armazena o nº do IP do Terminal (Ecd), no formato Host.
    WORD Port;  //!< Porta que será utilizada no Socket (TCP).
};

namespace func {

/**
* Inicializa o Servidor de ECDs. É o ponteiro (*void) da janela da aplicação.
*/
typedef int (*bInitialize)(HWND handle);

/**
 *  Finaliza o Servidor de ECDs.
 */
typedef int (*bTerminate) (void);

/**
 * Envia para o terminal com ip e porta o codigo do produto e o preco.
 */
typedef int (*SendPrice) (char *ip, unsigned int port, char *codigo, char *preco);

/**
 * Envia para o terminal com ip e porta as configuracoes (para terminal de 4 linhas).
 */
typedef int (*SendConfiguration) (char *ip, unsigned int port, char *ip_cliente, char *ip_server, char *mascara, char *linha1, char *linha2, char *linha3, char *linha4, char *tempo);

/**
 * Requisita para o terminal os valores das configuracoes. A forma retornada na string data ver no manual item 3.
 */
typedef int (*GetConfiguration) (char *ip, unsigned int port, char *data);

/**
 * Esta rotina sempre retornará TRUE, função contida na dll para manter compatibilidade.
 */
typedef int (*bShowWindow) (void);

/**
 * Permite ao aplicativo obter o número da mensagem registrada no Windows. Sempre chamar essa funcao apos passando bInitialize valor 0.
 *
 */
typedef void (*vGetMsgWinReceiveDataTC) (LPUINT MsgWinReg);


typedef int (*bReceiveBarcode) (stAddress *ID, char *barcode);
typedef int (*bSendProdNotFound) (stAddress  *ID);
typedef int (*bSendProdPriceTC50X) (stAddress *ID, char *NameProd, char *PriceProd);
typedef int (*bConnect) (stAddress *ID);
typedef int (*bDisconnect) (stAddress *ID);
typedef void (*Inet_NtoA) (DWORD nIP, char *sIP);
typedef DWORD (*Inet_Addr) (char *sIP);
typedef int (*SendConfigurationNew) (char *ip, unsigned int port, char *ip_cliente, char *ip_server, char *mascara, char *linha1, char *linha2, char *linha3, char *linha4, char *tempo, char *gateway, char *linha5, char *linha6, char *linha7, char *linha8, char *contraste, bool quieto, bool wifi, char *ssid, char *seguranca, char *senha);
typedef int (*GetEcdVersion) (char *ip, unsigned int port, char *data);
typedef int (*GetMacAddress) (char *ip, unsigned int port, char *data);
typedef int (*SetEcdVersion) (char *ip, unsigned int port, char *data);
typedef int (*TestEcd1200NewVersion) (char *ip, unsigned int port, char *data);
typedef int (*GetConfigurationNew) (char *ip, unsigned int port, char *ip_cliente, char *ip_server,char *mascara, char *linha1, char *linha2, char *linha3, char *linha4, char *tempo, char *gateway, char *linha5, char *linha6, char *linha7, char *linha8);
typedef int (*CloseTerminal) (stAddress *ID);
typedef int (*bSendPriceNew) (stAddress *ID , char *NameProd ,char *PriceProd, char *dadoslinha3 ,char *dadoslinha4);
}

EcdSweda::EcdSweda()
{
    library = new QLibrary(libraryName);

    if(!library->load()) {
        qDebug() << "erro ao carregar a biblioteca " + libraryName;
    }
}

EcdSweda::~EcdSweda()
{
    if(library->isLoaded())
        library->unload();

    delete library;
}

int EcdSweda::bInitialize(HWND handle)
{
    if(!library->isLoaded())
        return 0;

    func::bInitialize f_bInitialize = (func::bInitialize)library->resolve("bInitialize");

    if(f_bInitialize == 0)
        return 0;

    return f_bInitialize(handle);
}

void EcdSweda::vGetMsgWinReceiveDataTC(LPUINT msg)
{
    if(!library->isLoaded())
        return ;

    func::vGetMsgWinReceiveDataTC f_vGetMsgWinReceiveDataTC = (func::vGetMsgWinReceiveDataTC)library->resolve("vGetMsgWinReceiveDataTC");

    if(f_vGetMsgWinReceiveDataTC == 0)
        return ;

    return f_vGetMsgWinReceiveDataTC(msg);
}


int EcdSweda::bTerminate()
{
    if(!library->isLoaded())
        return 0;

    func::bTerminate f_bTerminate = (func::bTerminate)library->resolve("bTerminate");

    if(f_bTerminate == 0)
        return 0;

    return f_bTerminate();
}


int EcdSweda::bConnect(int &port, QString &ip) {
    if(!library->isLoaded())
        return 0;

    func::bConnect f_bConnect = (func::bConnect)library->resolve("bConnect");

    if(f_bConnect == 0) {
        qDebug() << "pointer f_bConnect is null";
        return 0;
    }

    stAddress ID;
    memset(&ID, 0, sizeof(ID));

    int response = f_bConnect(&ID);

    if(response == 0) {
        qDebug() << "f_bConnect return 0";
        return 0;
    }

    func::Inet_NtoA f_Inet_NtoA = (func::Inet_NtoA)library->resolve("Inet_NtoA");

    if(f_Inet_NtoA == 0) return 0;

    char ipStr[30] = { 0 };
    f_Inet_NtoA(ID.Ip, ipStr);

    port = ID.Port;
    ip = QString(ipStr);

    return response;
}

int EcdSweda::bDisconnect(int &port, QString &ip)
{
    if(!library->isLoaded())
        return 0;

    func::bDisconnect f_bDisconnect = (func::bDisconnect)library->resolve("bDisconnect");

    if(f_bDisconnect == 0) {
        qDebug() << "pointer f_bDisconnect is null";
        return 0;
    }

    stAddress ID;
    memset(&ID, 0, sizeof(ID));

    int response = f_bDisconnect(&ID);

    if(response == 0) {
        qDebug() << "f_bDisconnect return 0";
        return 0;
    }

    func::Inet_NtoA f_Inet_NtoA = (func::Inet_NtoA)library->resolve("Inet_NtoA");

    if(f_Inet_NtoA == 0) return 0;

    char ipStr[30] = { 0 };
    f_Inet_NtoA(ID.Ip, ipStr);

    port = ID.Port;
    ip = QString(ipStr);

    return response;
}

int EcdSweda::bReceiveBarcode(int &port, QString &ip, QString &barcode)
{
    if(!library->isLoaded())
        return 0;

    func::bReceiveBarcode f_bReceiveBarcode = (func::bReceiveBarcode)library->resolve("bReceiveBarcode");

    if(f_bReceiveBarcode == 0) {
        qDebug() << "pointer f_bReceiveBarcode is null";
        return 0;
    }

    stAddress ID;
    memset(&ID, 0, sizeof(ID));
    char barcodeStr[256] = { 0 };

    int response = f_bReceiveBarcode(&ID, barcodeStr);

    if(response == 0) {
        qDebug() << "f_bReceiveBarcode return 0";
        return 0;
    }


    func::Inet_NtoA f_Inet_NtoA = (func::Inet_NtoA)library->resolve("Inet_NtoA");

    if(f_Inet_NtoA == 0) return 0;

    char ipStr[30] = { 0 };
    f_Inet_NtoA(ID.Ip, ipStr);

    port = ID.Port;
    ip = QString(ipStr);
    barcode = QString(barcodeStr);

    return response;
}
